Archy = LibStub("AceAddon-3.0"):NewAddon("Archy", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "LibSink-2.0");
local ldb = LibStub("LibDataBroker-1.1"):NewDataObject("Archy", {
	type = "data source",
	icon = "Interface\\Icons\\trade_archaeology",
	text = "Archy",
	label = "Archy",
	version = "1.4",
	align = "right",
	["X-Category"] = "Information",
});
local icon = LibStub("LibDBIcon-1.0")
local deformat = LibStub("LibDeformat-3.0")
local QT = LibStub("LibQTip-1.0");
local tooltip;


local DEBUG_MODE = false;

function DebugPrint(...)
	if DEBUG_MODE then
		Archy:Print("DEBUG: " .. ...);
	end
end

RACES_LOADED = false;
RARE_ITEM_COLOR_CODE = "|cFF0070DD";
solvedArtifact = nil;
tomtomFrame = nil;
tomtompoint = nil;
minimapPool = {};
ARCHAEOLOGY_MAX_RACES = 12;
ARCHAEOLOGY_MAX_STONES = 4;
ARCHAEOLOGY_MAX_DIGSITES_PER_CONTINENT = 4;
REFRESH_ON_NEXT_BAG_UPDATE = false;
DIGSITE_TEXTURE_INDEX = 177;
LAST_DIGSITE = nil;
RACES = {}
CONTINENTS = {};
CONTINENTS[1] = "Kalimdor";
CONTINENTS[2] = "Eastern Kingdoms";
CONTINENTS[3] = "Outlands";
CONTINENTS[4] = "Northrend";

local ACE_OPTS = {
	name = "Archy",
	handler = Archy,
	type = "group",
	args = {
		toggle = {
			name = "Show/Hide Archy",
			desc = "Shows (or hides) the Archy displays",
			order = 1,
			guiHidden = true,
			type = "execute",
			func = "ToggleFrames"
		},
		lock = {
			name = "Locked",
			desc = "Locks (or unlocks) the Archys displays for repositioning",
			order = 2,
			type = "toggle",
			get = "GetLockState",
			set = "SetLockState"
		}, 
		reset = {
			name = "Reset Positions",
			desc = "Restores the Archy display positions to defaults",
			order = 3,
			type = "execute",
			func = "ResetFramePositions"			
		},
		refresh = {
			name = "Refresh",
			desc = "Refresh data and displays used and provided by Archy",
			order = 4,
			guiHidden = true,
			type = "execute",
			func = "Refresh"			
		},
		tomtom = {
			name = "TomTom Integration",
			desc = "Enable (or disable) Archy sending the nearest waypoint information to TomTom",
			order = 5,
			type = "toggle",
			get = "GetTomTomState",
			set = "SetTomTomState"
		},
		minimap = {
			name = "Show Digsites on Minimap",
			desc = "Show the digsites on your minimap",
			order = 6,
			type = "toggle",
			width = "full",
			get = "GetMinimapDigsiteState",
			set = "SetMinimapDigsiteState"
		},
		minimapIcon = {
			name = "Show minimap button",
			desc = "Toggles showing the minimap button",
			order = 7,
			type = "toggle", 
			width = "full",
			cmdHidden = true,
			get = "GetMinimapButtonState",
			set = "SetMinimapButtonState"
		},
		extraStats = {
			name = "Show extra stats",
			desc = "Show extra statistics in the minimap (or ldb) tooltip",
			order = 8,
			type = "toggle",
			width = "full",
			cmdHidden = true,
			get = "GetExtraStatsState",
			set = "SetExtraStatsState"
		},
		nearest = {
			name = "Nearest Digsite",
			desc = "Announces the nearest digsite",
			order = 7,
			guiHidden = true,
			type = "execute",
			func = "AnnounceNearestDigsite"
		},
		config = {
			name = "Config",
			desc = "Shows the configuration window",
			order = 8,
			guiHidden = true,
			type = "execute",
			func = "ShowConfig"
		}
		
	}
}

local DB_DEFAULTS = {
	global = {
		digsites = {},
		 
	}
}
function loadArchaeology()
	ShowUIPanel(ArchaeologyFrame);
	HideUIPanel(ArchaeologyFrame);
end 

function Archy:SetupAstrolabe()
	local orig = GetCurrentMapAreaID();
	for cid, zlist in ipairs{GetMapContinents()} do
		for zid, mapid in ipairs{GetMapZones(cid)} do
			SetMapZoom(cid, zid)
			local mapid = GetCurrentMapAreaID()
			Archy.mapcz[mapid] = {cid, zid}
		end
	end
	SetMapByID(orig);
	Archy.astrolabe.MinimapUpdateTime = 0.1;
end

function Archy:OnInitialize()
	DebugPrint("DEBUG: Archy:OnInitialize");
	self.db = LibStub("AceDB-3.0"):New("ArchyDB");
	self.astrolabe = DongleStub("Astrolabe-1.0");
	self.mapcz = {};
	self:SetupAstrolabe();
	
	self.myDigsites = {};
	self.myArtifacts = {};
	
	if not self.db.char.digsites then
		self.db.char.digsites = {};
	end
	if not self.db.char.artifacts then
		self.db.char.artifacts = {};
	end
	
	if not self.db.char.blacklist then
		self.db.char.blacklist = {};
	end
	
	if self.db.char.showExtraStats == nil then
		self.db.char.showExtraStats = false;
	end
	
	if not self.db.char.icon then
		self.db.char.icon = { hide = false };
	end
	if self.db.char.hidden == nil then
		self.db.char.hidden = false;
	end
	if self.db.char.locked == nil then
		self.db.char.locked = false;
	end
	if self.db.char.useTomTom == nil then
		self.db.char.useTomTom = false;
	end
	if self.db.char.showDigsitesOnMinimap == nil then
		self.db.char.showDigsitesOnMinimap = true;
	end
	
	if self.db.global.digsites == nil then
		self.db.global.digsites = {};
	end
	
	self.db.char.frames = self.db.char.frames or { digsites = { x = 0, y = 0 }, artifacts = { x = 0, y = 0 }};
	self.db.char.frames.digsites = self.db.char.frames.digsites or { x = 0, y = 0 };
	self.db.char.frames.artifacts = self.db.char.frames.artifacts or { x = 0, y = 0 };
	
	
	self.keystoneIds = {};
	
	self.frames = {};
	self.frames.digsites = CreateFrame("Frame", "Digsites", UIParent, "ArchyDigsiteListFrame");
	self.frames.artifacts = CreateFrame("Frame", "Artifacts", UIParent, "ArchyArtifactListFrame");
	
	self.frames.digsites:ClearAllPoints();
	self.frames.artifacts:ClearAllPoints();
	self.frames.digsites:SetPoint("CENTER", UIParent, "CENTER", self.db.char.frames.digsites.x, self.db.char.frames.digsites.y);
	self.frames.artifacts:SetPoint("CENTER", UIParent, "CENTER", self.db.char.frames.artifacts.x, self.db.char.frames.artifacts.y);

	icon:Register("Archy", ldb, Archy.db.char.icon)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Archy", ACE_OPTS, { "archy", "arch" });
	self.options = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Archy", "Archy");
	self:ScheduleTimer("Refresh", 2);
end


function Archy:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.options);
end

function Archy:GetLockState()
	return self.db.char.locked;
end
function Archy:SetLockState(info, value)
	if value then
		self:Lock();
	else
		self:Unlock();
	end
end


function Archy:GetExtraStatsState()
	return self.db.char.showExtraStats;
end
function Archy:SetExtraStatsState(info, value)
	self.db.char.showExtraStats = value;
end

function Archy:GetMinimapButtonState()
	return not self.db.char.icon.hide;
end
function Archy:SetMinimapButtonState(info, value)
	self.db.char.icon.hide = not value;
	if self.db.char.icon.hide then
		icon:Hide("Archy")
	else
		icon:Show("Archy")
	end
end

function Archy:GetMinimapDigsiteState()
	return self.db.char.showDigsitesOnMinimap;
end
function Archy:SetMinimapDigsiteState(info, value)
	self.db.char.showDigsitesOnMinimap = value;
	if self.db.char.showDigsitesOnMinimap then
		self:Print("Show digsites on the minimap");
	else
		self:Print("Hide digsites on the minimap");
	end
	self:RefreshMinimapIcons();
end


function populateRaceList()
	if GetNumArchaeologyRaces() == 0 then
		return;
	end
	local numRaces = GetNumArchaeologyRaces();
	for i=1,ARCHAEOLOGY_MAX_RACES do
		local name, currency, texture, itemID =  GetArchaeologyRaceInfo(i);
		RACES[name] = i;
		Archy.keystoneIds["" .. itemID] = itemID;
	end
	RACES_LOADED = true;
end

function Archy:OnEnable()
	DebugPrint("DEBUG: Archy:OnEnable");
	local name = GetArchaeologyInfo();
	local hasArch, rank, maxRank = GetArchaeologySkillInfo();
	if not self.hasArch then
		self.db.char.hidden = true;
		DebugPrint("Does NOT have Archaeology");
	else
		DebugPrint("Does have Archaeology");
	end
	
	self:RegisterEvent("SKILL_LINES_CHANGED");

	if not self.db.char.hidden then
		self.frames.digsites:Show();
		self.frames.artifacts:Show();
	else 
		self.frames.digsites:Hide();
		self.frames.artifacts:Hide();
	end
	
	if self.db.char.useTomTom then
		Archy:EnableTomTom();
	end
end

function Archy:SKILL_LINES_CHANGED()
	self:UpdateLDB();
end


function Archy:OnDisable()
	DebugPrint("DEBUG: Archy:OnDisable");
end

function Archy:Refresh()
	if GetNumArchaeologyRaces() == 0 then
		return;
	end
	DebugPrint("Refresh");
	populateRaceList();
	Archy:RefreshDigsites();
	Archy:RefreshArtifacts();
end

function Archy:RefreshArtifacts()
	self:UpdateArtifacts();
	ArchyArtifactListFrame_Update(self.frames.artifacts);
end

function Archy:RefreshDigsites()
	self:UpdateDigsites();
	ArchyDigsiteListFrame_Update(self.frames.digsites);
end


function Archy:ToggleFrames()
	self.db.char.hidden = not self.db.char.hidden;
	if self.db.char.hidden then
		self.frames.artifacts:Hide();
		self.frames.digsites:Hide();
		if self.db.char.useTomTom then
			self:DisableTomTom();
		end
	else
		self.frames.artifacts:Show();
		self.frames.digsites:Show();
		if self.db.char.useTomTom then
			self:EnableTomTom();
		end
	end
end

function Archy:AnnounceNearestDigsite()
	self:Print("Nearest digsite is: " .. self:NearestDigsiteName());
end

function Archy:ResetFramePositions()
	self:Print("Frame positions have been reset");
	self.frames.digsites:ClearAllPoints();
	self.frames.digsites:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	self.frames.artifacts:ClearAllPoints();
	self.frames.artifacts:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
end

function Archy:ToggleLock()
	if self.db.char.locked then
		self:Unlock();
	else
		self:Lock();
	end
end

function Archy:Unlock()
	self:Print("Unlocked");
	self.db.char.locked = false;
end

function Archy:Lock()
	self:Print("Locked");
	self.db.char.locked = true;
end


function Archy:GetTomTomState()
	return self.db.char.useTomTom;
end
function Archy:SetTomTomState(info, value)
	self.db.char.useTomTom = value;
	if self.db.char.useTomTom then
		self:Print("Enabling TomTom integration");
		if not tomtomFrame then
			self:EnableTomTom();
		else
			tomtomFrame:Show();
		end
	else
		self:Print("Disabling TomTom integration");
		Archy:DisableTomTom();
	end
end

function Archy:DisableTomTom()
	if TomTom and TomTom.AddZWaypoint and TomTom.RemoveWaypoint then
--		DebugPrint("TomTom exists");
		if tomtomFrame then
			if tomtompoint then
				tomtompoint = TomTom:RemoveWaypoint(tomtompoint);
				tomtompoint = nil;
			end
			tomtomFrame:Hide();
			tomtomFrame:SetScript("OnUpdate", nil);
			tomtomFrame = nil;
		end
	end
end

function Archy:EnableTomTom()
	if TomTom and TomTom.AddZWaypoint and TomTom.RemoveWaypoint then
--		DebugPrint("TomTom exists");
		if not tomtomFrame then
			local t = 0
			tomtomFrame = CreateFrame("Frame");
			tomtomFrame:SetScript("OnUpdate", function(self, elapsed)
				t = t + elapsed
				if t > 3 then	-- this doesn't change very often at all; maybe more than 3 seconds?
					t = 0;
					if tomtompoint then
						tomtompoint = TomTom:RemoveWaypoint(tomtompoint);
					end
					local closest = Archy:NearestDigsite();
					if closest and closest.title and closest.title ~= "" then
						--DebugPrint("Attempting to add ZWaypoint: " .. closest.title .. " C: " .. closest.c .. " Z: " .. closest.z .. " X: " .. closest.x .. " Y: " .. closest.y);
						tomtompoint = TomTom:AddZWaypoint(closest.c, closest.z, closest.x * 100, closest.y * 100, closest.title, false, false, false, false, false, true);
					end
				end
			end)
		end
		tomtomFrame:Show();
	else
		self:Print("TomTom was not found.");
		self.db.char.useTomTom = false;
	end
end

function Archy:GetMapIDFloor(c, z)
	local mapId = Archy.astrolabe:GetMapID(c, z);
	local floors = Archy.astrolabe:GetNumFloors(mapId);
	local fl = floors == 0 and 0 or 1;
	return mapId, fl;
end

function Archy:UpdateDigsites()
	if not RACES_LOADED then
		return;
	end
	DebugPrint("Archy:UpdateDigsites");
--	print(debugstack(1));
	
	if self.db.char.useTomTom then
		self:EnableTomTom();
	end
	
	for continentID=1,4 do
		if not Archy.myDigsites[continentID] then
			Archy.myDigsites[continentID] = {};
		end
		
		SetMapZoom(continentID);
		local totalPOIs = GetNumMapLandmarks();
		local digsiteNum = 1;
		for index = 1,totalPOIs do
			local worldMapPOIName = "WorldMapFramePOI"..index;
			local worldMapPOI = _G[worldMapPOIName];
			if index <= totalPOIs then
				local name, description, textureIndex, px, py, mapLinkID, inBattleMap, graveyardID, areaID = GetMapLandmarkInfo(index);
				if textureIndex == DIGSITE_TEXTURE_INDEX then
					zoneName, fileName, texPctX, texPctY, texX, texY, scrollX, scrollY = UpdateMapHighlight(px, py);


					local raceID = 0
					if Archy.db.global.digsites[name] then
						raceID = Archy.db.global.digsites[name];
					end

					local raceName, currency, texture, itemID = GetArchaeologyRaceInfo(raceID)

					local md, fd = Archy:GetMapIDFloor(continentID, 0);
					local zoneID = Archy_GetZoneID(zoneName);
					local mz, fz = Archy:GetMapIDFloor(continentID, zoneID);
					
					nx, ny = Archy.astrolabe:TranslateWorldMapPosition(md, fd, px, py, mz, fz);

					Archy.myDigsites[continentID][digsiteNum] = {
						zoneName = zoneName,
						zoneID = zoneID,
						name = name,
						raceName = raceName,
						raceTexture = texture,
						raceID = raceID,
						waypoint = { c = continentID, z = zoneID, m = mz, f = fz, x = nx, y = ny },
						minimapIcon = Archy:CreateMinimapIcon(name),
					};

					digsiteNum = digsiteNum + 1
				end
			end
		end
		SetMapToCurrentZone()
	end
	self:RefreshMinimapIcons();
end

function Archy:UpdateArtifacts()
	if not RACES_LOADED then
		return;
	end
	DebugPrint("Archy:UpdateArtifacts");
	if not Archy.hasArch then
		return;
	end
	
	local numRaces = GetNumArchaeologyRaces();
	local raceFrame;
	for i=1,ARCHAEOLOGY_MAX_RACES do
		if i <= numRaces then
			local raceName, currency, texture, itemID = GetArchaeologyRaceInfo(i);
			local numProjects = GetNumArtifactsByRace(i)
			
			if numProjects and numProjects > 0 then
				
				local name, description, rarity, icon, spellDescription, numSockets, bgTexture =  GetActiveArtifactByRace(i)
				SetSelectedArtifact(i)
				local base, adjust, totalCost = GetArtifactProgress()
				
				numSockets = numSockets or 0;
				local keystones = 0;
				local keystoneName = nil;
				
				
				if itemID and itemID > 0 then
					local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemID);
					keystoneName = itemName;
					keystones = GetItemCount(itemID);
				end

				local artifact = {};
				artifact.raceID = i;
				artifact.raceName = raceName;
				artifact.raceTexture = texture;
				artifact.keystoneItemID = itemID;
				artifact.name = name;
				artifact.description = description;
				artifact.rarity = rarity;
				artifact.icon = icon;
				artifact.spellDescription = spellDescription;
				artifact.numSockets = numSockets;
				artifact.bgTexture = bgTexture;
				artifact.progress = base;
				artifact.keystoneAdjust = adjust;
				artifact.totalCost = totalCost;
				artifact.keystones = keystones;
				artifact.keystoneName = keystoneName;
				Archy.myArtifacts[i] = artifact;
			else
				if Archy.myArtifacts[i] then
					Archy.myArtifacts[i] = nil;
				end
			end
		end
	end
end

function Archy:RefreshMinimapIcons()
	self:RemoveMinimapIcons();
	self:AddMinimapIcons();
end

function Archy:RemoveMinimapIcons()
	for _,icon in ipairs(minimapPool) do
		Archy.astrolabe:RemoveIconFromMinimap(icon);
	end
end

function Archy:AddMinimapIcons()
	if Archy.db.char.showDigsitesOnMinimap then
		if Archy.myDigsites[GetCurrentMapContinent()] then
			for _,digsite in pairs(self.myDigsites[GetCurrentMapContinent()]) do
				Archy.astrolabe:PlaceIconOnMinimap(digsite.minimapIcon, digsite.waypoint.m, digsite.waypoint.f, digsite.waypoint.x, digsite.waypoint.y);
			end
		end
	end
end

function Archy:CreateMinimapIcon(name)
	local minimap = CreateFrame("Button", nil, Minimap);
	minimap:SetSize(14,14);
	minimap:RegisterForClicks("RightButtonUp");
	minimap.name = name;
	minimap.kind = "MinimapIcon";
	
	minimap.icon = minimap:CreateTexture(nil, "OVERLAY");
	minimap.icon:SetTexture("Interface\\Archeology\\Arch-Icon-Marker.blp");
	minimap.icon:SetPoint("CENTER", 0, 0);
	minimap.icon:SetSize(14, 14);
	
	minimap:SetScript("OnEnter", OnMinimapEnter);
	minimap:SetScript("OnLeave", OnMinimapLeave);
	minimap:SetScript("OnClick", OnMinimapClick);
	minimap:SetScript("OnUpdate", OnMinimapUpdate);

	tinsert(minimapPool, minimap);

	return minimap;
end

function OnMinimapEnter(self)
	GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT");
	GameTooltip:SetText(self.name);
	GameTooltip:Show();
end

function OnMinimapLeave(self)
	GameTooltip:Hide();
end

function OnMinimapClick(self, button)
	if (button ~= "RightButton") then
		return;
	else
		-- 
	end
end

function Archy:NearestDigsiteName(arg)
	local closest = self:NearestDigsite(arg);
	return closest.title or "";
end			

function Archy:GetContinentZone(mapID, floorNum)
	local cz = Archy.mapcz[mapID];
	if cz then
		local c, z = unpack(cz);
		return c, z;
	end
	return 0, 0;
end

function Archy:GetCurrentPlayerPosition()
	local m, f, x, y = Archy.astrolabe:GetCurrentPlayerPosition();
	local cz = Archy.mapcz[m];
	if cz then
		local c, z = unpack(cz);
		return m, f, c, z, x, y;
	end
end

function Archy:NearestDigsite(arg)
	local distance
	local closest = { title = "", x = 0, y = 0, z = 0, c = 0, m = 0, f = 0 };
	local m, f, c, z, x, y = Archy:GetCurrentPlayerPosition();
			
	local anySite = false;
	if arg and arg.anySite then
		anySite = true;
	end
	
	if Archy.myDigsites[c] then
		for _,digsite in pairs(Archy.myDigsites[c]) do
			if (not Archy.db.char.blacklist[digsite.name] and not anySite) or (anySite) then
				local dist, xDelta, yDelta = Archy.astrolabe:ComputeDistance(m, f, x, y, digsite.waypoint.m, digsite.waypoint.f, digsite.waypoint.x, digsite.waypoint.y);
				if distance then
					if dist < distance then
						distance = dist;
						closest.title = digsite.name;
						closest.c = digsite.waypoint.c;
						closest.z = digsite.waypoint.z;
						closest.x = digsite.waypoint.x;
						closest.y = digsite.waypoint.y;
						closest.m = digsite.waypoint.m;
						closest.f = digsite.waypoint.f;
					end
				else
					distance = dist;
					closest.title = digsite.name;
					closest.c = digsite.waypoint.c;
					closest.z = digsite.waypoint.z;
					closest.x = digsite.waypoint.x;
					closest.y = digsite.waypoint.y;
					closest.m = digsite.waypoint.m;
					closest.f = digsite.waypoint.f;
				end
			end
		end
	end
	
	return closest;
end

-- Data Broker --
function ldb:OnEnter()
	if not QT then
		return;
	end
	tooltip = QT:Acquire("ArchyTT", 5, "LEFT", "RIGHT", "RIGHT", "RIGHT", "RIGHT");
	tooltip:SetScale(1)
	tooltip:SetAutoHideDelay(0.1, self);
	Archy:DrawTooltip();
	tooltip:EnableMouse();
	tooltip:SmartAnchorTo(self);
	tooltip:UpdateScrolling();
	tooltip:Show();
	
	
end


function Archy:DrawTooltip()
		
	local plusIcon = "|TInterface\\Buttons\\UI-PlusButton-Up:18:18:1:0|t"
	local minusIcon = "|TInterface\\Buttons\\UI-MinusButton-Up:18:18:1:0|t"

	tooltip:Hide();
	tooltip:Clear();
	
	local t = {};
	local numCols = 5;
	t[1] = "LEFT";
	t[2] = "LEFT";
	t[3] = "LEFT";
	t[4] = "LEFT";
	t[5] = "LEFT";
	if Archy.db.char.showExtraStats then
		numCols = numCols + 4;
	end
	
	
	tooltip:SetColumnLayout(numCols, unpack(t));
	
	local lineNum = tooltip:AddHeader("b");
	tooltip:SetCell(lineNum, 1, ORANGE_FONT_COLOR_CODE .. "Archy|r", "CENTER", numCols);
	local colIndex = 0;
	local t = {};
	
	-- Artifacts
	if Archy.hasArch then
		lineNum = tooltip:AddLine(" ");
		
		local hasArch, rank, maxRank = GetArchaeologySkillInfo();
		local skill = format("%s/%s", rank, maxRank);
		if maxRank < 525 and (maxRank - rank) <= 25 then
			skill = skill .. " - |cFFFF0000Visit a trainer!|r";
		end
		tooltip:SetCell(lineNum, 1, NORMAL_FONT_COLOR_CODE .. "Skill:|r " .. skill, "CENTER", numCols);
		
		lineNum = tooltip:AddLine(" ");
		tooltip:SetCell( lineNum, 1, "|cFFFFFF00Artifacts|r", "LEFT", numCols);
	
		lineNum = tooltip:AddLine("b");
		tooltip:SetCell(lineNum, 1, " ", "LEFT", 1);
		tooltip:SetCell(lineNum, 2, NORMAL_FONT_COLOR_CODE .. "Race|r", "LEFT", 1);
		if Archy.db.char.showExtraStats then
			tooltip:SetCell(lineNum, 3, NORMAL_FONT_COLOR_CODE .. "Artifact|r", "LEFT", 2);
			tooltip:SetCell(lineNum, 6, NORMAL_FONT_COLOR_CODE .. "Progress|r", "CENTER", 2);
			tooltip:SetCell(lineNum, 8, NORMAL_FONT_COLOR_CODE .. "Keystones|r", "CENTER", 2);
		else
			tooltip:SetCell(lineNum, 3, NORMAL_FONT_COLOR_CODE .. "Artifact|r", "LEFT", 1);
			tooltip:SetCell(lineNum, 4, NORMAL_FONT_COLOR_CODE .. "Progress|r", "LEFT", 1);
			tooltip:SetCell(lineNum, 5, NORMAL_FONT_COLOR_CODE .. "Keystones|r", "LEFT", 1);
		end

		for raceID,artifact in pairs(Archy.myArtifacts) do
			lineNum = tooltip:AddLine("b");
			tooltip:SetCell(lineNum, 1, " " .. format("|T%s:18:18:0:1:128:128:4:60:4:60|t", artifact.raceTexture), "LEFT", 1);
			tooltip:SetCell(lineNum, 2, artifact.raceName, "LEFT", 1);
			local artifactName = artifact.name;
			if artifact.rarity == 0 then
				artifactName = GRAY_FONT_COLOR_CODE .. artifactName .. "|r";
			else
				artifactName = RARE_ITEM_COLOR_CODE .. artifactName .. "|r";
			end
		
			local progress;
			if artifact.progress >= artifact.totalCost then
				progress = "|cFF00FF00" .. artifact.progress .. "|r";
			else
				progress = "|cFFFFFFFF" .. artifact.progress .. "|r";
			end
			progress = progress .. "/|cFFFFFFFF" .. artifact.totalCost .. "|r";

			local keystones = format("|cFFFFFF00%s/%s|r", artifact.keystones, artifact.numSockets);
			if artifact.numSockets == 0 and artifact.keystones == 0 then
				keystones = "";
			end
			if self.db.char.showExtraStats then
				tooltip:SetCell(lineNum, 3, artifactName, "LEFT", 2);
				tooltip:SetCell(lineNum, 6, progress, "CENTER", 2);
				tooltip:SetCell(lineNum, 8, keystones, "CENTER", 2);
			else
				tooltip:SetCell(lineNum, 3, artifactName, "LEFT", 1);
				tooltip:SetCell(lineNum, 4, progress, "CENTER", 1);
				tooltip:SetCell(lineNum, 5, keystones, "CENTER", 1);
			end
		
			if self.db.char.showExtraStats then
			
			end
		end
	
		-- Digsites
		tooltip:AddLine(" ");
		lineNum = tooltip:AddLine(" ");
		tooltip:SetCell( lineNum, 1, "|cFFFFFF00Digsites|r", "LEFT", numCols);

		for continentID,continentDigsites in pairs(Archy.myDigsites) do
			lineNum = tooltip:AddLine("b");
			tooltip:SetCell(lineNum, 1,"  " .. ORANGE_FONT_COLOR_CODE .. CONTINENTS[continentID] .. "|r", "LEFT", numCols);
		
			lineNum = tooltip:AddLine("b");
			tooltip:SetCell(lineNum, 1, " ", "LEFT", 1);
			tooltip:SetCell(lineNum, 2, NORMAL_FONT_COLOR_CODE .. "Fragment|r", "LEFT", 1);
			tooltip:SetCell(lineNum, 3, NORMAL_FONT_COLOR_CODE .. "Digsite|r", "LEFT", 1);
			tooltip:SetCell(lineNum, 4, NORMAL_FONT_COLOR_CODE .. "Zone|r", "LEFT", 2);
			if self.db.char.showExtraStats then
				tooltip:SetCell(lineNum, 6, NORMAL_FONT_COLOR_CODE .. "Surveys|r", "LEFT", 1);
				tooltip:SetCell(lineNum, 7, NORMAL_FONT_COLOR_CODE .. "Looted|r", "LEFT", 1);
				tooltip:SetCell(lineNum, 8, NORMAL_FONT_COLOR_CODE .. "Frags|r", "LEFT", 1);
				tooltip:SetCell(lineNum, 9, NORMAL_FONT_COLOR_CODE .. "Keys|r", "LEFT", 1);
			end
		
			for _, digsite in pairs(continentDigsites) do
				local raceName = digsite.raceName;
				if raceName == "UNKNOWN" then
					raceName = " ";
				end
				lineNum = tooltip:AddLine("b");
				tooltip:SetCell(lineNum, 1, " " .. format("|T%s:18:18:0:1:128:128:4:60:4:60|t", digsite.raceTexture), "LEFT", 1);
				tooltip:SetCell(lineNum, 2, raceName, "LEFT", 1);
				tooltip:SetCell(lineNum, 3, digsite.name, "LEFT", 1);
				tooltip:SetCell(lineNum, 4, digsite.zoneName, "LEFT", 2);
				if self.db.char.showExtraStats then
					if Archy.db.char.digsites[digsite.name] then
						tooltip:SetCell(lineNum, 6, Archy.db.char.digsites[digsite.name].surveys, "CENTER", 1);
						tooltip:SetCell(lineNum, 7, Archy.db.char.digsites[digsite.name].looted, "CENTER", 1);
						tooltip:SetCell(lineNum, 8, Archy.db.char.digsites[digsite.name].fragments, "CENTER", 1);
						tooltip:SetCell(lineNum, 9, Archy.db.char.digsites[digsite.name].keystones, "CENTER", 1);
					end
				end
			end
			tooltip:AddLine(" ");
		end
	else
		lineNum = tooltip:AddLine(" ");
		tooltip:SetCell( lineNum, 1, "Learn Archaeology in your nearest major city!", "CENTER", numCols);
	end
	
	tooltip:AddLine(" ");
	lineNum = tooltip:AddLine("c");
	tooltip:SetCell(lineNum, 1, "|cFF00FF00Left-Click to toggle Archy|r", "LEFT", numCols);
	lineNum = tooltip:AddLine("d");
	tooltip:SetCell(lineNum, 1, "|cFF00FF00Right-Click to lock/unlock Archy|r", "LEFT", numCols);
	lineNum = tooltip:AddLine("e");
	tooltip:SetCell(lineNum, 1, "|cFF00FF00Middle-Click to display the Archaeology window|r", "LEFT", numCols);
end

function ldb:OnLeave()
--	GameTooltip:Hide();
end

function ldb:OnClick(button)
	if button == "LeftButton" then
		Archy:ToggleFrames();
	elseif button == "RightButton" then
		if Archy.db.char.locked then
			Archy:Unlock();
		else
			Archy:Lock();
		end
	elseif button == "MiddleButton" then
		ShowUIPanel(ArchaeologyFrame);
	end
end


function ParseLootMessage(msg)
  local player = UnitName("player")
  local item, quantity = deformat(msg, LOOT_ITEM_SELF_MULTIPLE)
  if item and quantity then
    return player, item, tonumber(quantity)
  end
  quantity = 1
  item = deformat(msg, LOOT_ITEM_SELF)
  if item then
    return player, item, tonumber(quantity)
  end

  player, item, quantity = deformat(msg, LOOT_ITEM_MULTIPLE)
  if player and item and quantity then
    return player, item, tonumber(quantity)
  end

  quantity = 1
  player, item = deformat(msg, LOOT_ITEM)
  return player, item, tonumber(quantity)
end

function Archy:UpdateLDB()
	local hasArch, rank, maxRank = GetArchaeologySkillInfo();
	if hasArch then
		ldb.text = format("%s/%s", rank, maxRank);
		if maxRank < 525 and (maxRank - rank) <= 25 then
			ldb.text = "|cFFFF0000" .. "Visit a trainer!";
		end
		
		
	else
		ldb.text = "Learn Archaeology";
		
	end
	
end

function GetArchaeologySkillInfo()
	local _, _, arch = GetProfessions();
	if arch then
		if Archy then
			Archy.hasArch = true;
		end
		local name, texture, rank, maxRank = GetProfessionInfo(arch);
		return true, rank, maxRank;
	else
		if Archy then
			Archy.hasArch = false;
		end
		return false, nil, nil;
	end
end