
function ArchyArtifactListFrame_OnLoad(self)
	DebugPrint("Artifact List Frame Loaded");
	self:RegisterEvent("ARTIFACT_UPDATE");
	self:RegisterEvent("ARTIFACT_HISTORY_READY");
	self:RegisterEvent("ARTIFACT_COMPLETE");
	self:RegisterEvent("ARTIFACT_DIG_SITE_UPDATED");
	self:RegisterEvent("CURRENCY_DISPLAY_UPDATE");
--	self:RegisterEvent("SKILL_LINES_CHANGED");
	self:RegisterEvent("BAG_UPDATE");
end
			
function ArchyArtifactListFrame_OnEvent(self, event, ...)
	if event == "ARTIFACT_DIG_SITE_UPDATED" then
		Archy:UpdateArtifacts();
		ArchyArtifactListFrame_Update(self);
		
	elseif event == "BAG_UPDATE" then
		if REFRESH_ON_NEXT_BAG_UPDATE then
			Archy:UpdateArtifacts();
			ArchyArtifactListFrame_Update(self);
			REFRESH_ON_NEXT_BAG_UPDATE = false;
		end
		
		
	elseif event == "ARTIFACT_COMPLETE" then
		if solvedArtifact then
			if not Archy.db.char.artifacts[solvedArtifact.raceID] then
				Archy.db.char.artifacts[solvedArtifact.raceID] = {};
			end
			if not Archy.db.char.artifacts[solvedArtifact.raceID][solvedArtifact.name] then
				Archy.db.char.artifacts[solvedArtifact.raceID][solvedArtifact.name] = { rarity = solvedArtifact.rarity, count = 1 };
			else
				Archy.db.char.artifacts[solvedArtifact.raceID][solvedArtifact.name].count = Archy.db.char.artifacts[solvedArtifact.raceID][solvedArtifact.name].count + 1;
			end
			Archy:Print("Completed "..solvedArtifact.raceName.." artifact: " .. solvedArtifact.name .. " (x" .. Archy.db.char.artifacts[solvedArtifact.raceID][solvedArtifact.name].count .. " times)");
			solvedArtifact = nil;
		end
		Archy:UpdateArtifacts();
		ArchyArtifactListFrame_Update(self);
	
	elseif event == "CURRENCY_DISPLAY_UPDATE" then
		Archy:UpdateArtifacts();
		ArchyArtifactListFrame_Update(self);
	end
end
			
function ArchyArtifactListFrame_OnShow(self)
	DebugPrint("ArchyArtifactListFrame_OnShow");
	Archy:UpdateArtifacts();
	ArchyArtifactListFrame_Update(self);
end
			
function ArchyArtifactListFrame_OnHide(self)
end


function ArchyArtifactListFrame_Update(self)
	if not RACES_LOADED then
		return;
	end
	DebugPrint("Updating Artifact List");
	
	local currFrameNum = 1;
	
	for i=1,ARCHAEOLOGY_MAX_RACES do
		self["race"..i]:Hide();
		if Archy.myArtifacts[i] then
			local artifact = Archy.myArtifacts[i];
			raceFrame = self["race"..currFrameNum];
			raceFrame.raceID = artifact.raceID;
			raceFrame.raceName = artifact.raceName;
			
			local base = artifact.progress;
			if artifact.progress >= artifact.totalCost then
				base = "|cFF00FF00" .. base;
			end
			
			raceFrame.progress:SetText(base.."/"..artifact.totalCost);
			raceFrame.sockets.sockets = artifact.numSockets;
			raceFrame.sockets.keystones = artifact.keystones;
			raceFrame.sockets.keystoneName = artifact.keystoneName;
			
			if raceFrame.sockets.sockets > 0 or raceFrame.sockets.keystones > 0 then
				raceFrame.sockets.text:SetText(raceFrame.sockets.keystones .. "/" .. raceFrame.sockets.sockets);
				raceFrame.sockets:Show();
			else
				raceFrame.sockets:Hide();
			end
			
			if artifact.raceTexture and artifact.raceTexture ~= "" then
				raceFrame.solveButton:SetNormalTexture(artifact.raceTexture);
				raceFrame.solveButton:SetHighlightTexture(artifact.raceTexture);
			end
			local name = artifact.name;
			if name then
				if artifact.rarity ~= 0 and artifact.name then
					name = RARE_ITEM_COLOR_CODE .. name .. "|r";
				else
					name = GRAY_FONT_COLOR_CODE .. name .. "|r";
				end
			end
			raceFrame.artifactButton.artifactName:SetText(name);
			raceFrame:Show();
			currFrameNum = currFrameNum + 1;
		end
	end
end

function Archy_SolveArtifact(raceID, useKeystones)
	local raceName, currency, texture, itemID = GetArchaeologyRaceInfo(raceID);
	local numProjects = GetNumArtifactsByRace(raceID)
	if numProjects and numProjects > 0 then
		local name, description, rarity, icon, spellDescription, numSockets, bgTexture =  GetActiveArtifactByRace(raceID)
		SetSelectedArtifact(raceID)
		local base, adjust, totalCost = GetArtifactProgress()
		
		if numSockets and numSockets > 0 then
			for socketNum=1,numSockets do
				if useKeystones and not ItemAddedToArtifact(itemID) then
					SocketItemToArtifact();
				elseif (not useKeystones) and ItemAddedToArtifact(itemID) then
					RemoveItemFromArtifact();
				end
			end
		end

		local base, adjust, totalCost = GetArtifactProgress()
		Archy:Print("Trying to solve " .. raceName .. " Artifact: " .. name .. " (Fragments: " .. min(base + adjust) .. " of " .. totalCost .. ")");
						
		if CanSolveArtifact() then
			solvedArtifact = { name = name, rarity = rarity, raceID = raceID, raceName = raceName };
			SolveArtifact();
			REFRESH_ON_NEXT_BAG_UPDATE = true;
		end
	else
		Archy:Print("No artifact available to solve for " .. raceName);
		return;
	end		
end