local mapData = {};


function ArchyDigsiteListFrame_OnLoad(self)
	DebugPrint("Digsite List Frame Loaded");
	self:RegisterEvent("ARTIFACT_DIG_SITE_UPDATED");
	self:RegisterEvent("CHAT_MSG_LOOT");
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED");
	self:RegisterEvent("PLAYER_ENTERING_WORLD");
end

function Archy_GetZoneID(zoneName)
--	DebugPrint("Getting ZoneID for: " .. zoneName);
	for i=1, select("#", GetMapZones(GetCurrentMapContinent())), 1 do
		local zone = select(i, GetMapZones(GetCurrentMapContinent()));
		if zone == zoneName then
--			DebugPrint("Found zoneID for: " .. zoneName .. " = " .. i);
			return i;
		end
	end
--	DebugPrint("Did not find zoneID for: " .. zoneName);
	return nil;
end
			
function ArchyDigsiteListFrame_OnEvent(self, event, ...)
	if event == "ARTIFACT_DIG_SITE_UPDATED" then
		Archy:UpdateDigsites();
		ArchyDigsiteListFrame_Update(self);
		
	elseif event == "CHAT_MSG_LOOT" then
		local msg = ...
		fragment_type, amount = strmatch(msg, "You receive currency: (.+) Archaeology Fragment x(%d+)")
		if fragment_type then
			local zone = GetZoneText()
			local subzone = GetSubZoneText()
			
			-- Add Digsite to DB if it hasn't been discovered yet
			if not Archy.db.global.digsites[LAST_DIGSITE] then
				Archy.db.global.digsites[LAST_DIGSITE] = RACES[fragment_type];
--				Archy:Print("Just dug at: " .. LAST_DIGSITE .. " contains " .. fragment_type .. " fragments");
			end

			Archy.db.char.digsites[LAST_DIGSITE].looted = Archy.db.char.digsites[LAST_DIGSITE].looted + 1;
			Archy.db.char.digsites[LAST_DIGSITE].fragments = Archy.db.char.digsites[LAST_DIGSITE].fragments + amount;
			Archy:Refresh();

		else
			if string.find(msg, "You receive loot") then
				local _, itemLink, quantity = ParseLootMessage(msg);
				local _, _, Color, Ltype, itemID, Enchant, Gem1, Gem2, Gem3, Gem4, Suffix, Unique, LinkLvl, itemName = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
	
			  --Archy:Print("Item: " .. itemLink .. " (" .. itemID .. ") x " .. quantity);

			  if Archy.keystoneIds[itemID] then

				Archy.db.char.digsites[LAST_DIGSITE].keystones = Archy.db.char.digsites[LAST_DIGSITE].keystones + quantity;

				Archy:RefreshArtifacts();
			  end
	      	end
		end

		
	elseif event == "UNIT_SPELLCAST_SUCCEEDED" then
		unit, spell, _, _ = ...
		if unit == "player" and spell == "Survey" then
			LAST_DIGSITE = Archy:NearestDigsiteName{anySite=1};
			local nearest = Archy:NearestDigsiteName();
			if nearest == LAST_DIGSITE then
				if Archy.db.char.useTomTom and not Archy.db.char.blacklist[LAST_DIGSITE] then
					Archy:DisableTomTom();
				end
			end
			
			if not Archy.db.char.digsites[LAST_DIGSITE] then
				Archy.db.char.digsites[LAST_DIGSITE] = { surveys = 0, fragments = 0, looted = 0, keystones = 0 };
			end
			Archy.db.char.digsites[LAST_DIGSITE].surveys = Archy.db.char.digsites[LAST_DIGSITE].surveys + 1;
			
			--Archy:Print("LAST_DIGSITE: "..LAST_DIGSITE);
		end
			
	elseif event == "PLAYER_ENTERING_WORLD" then
		Archy:UpdateDigsites();
		if Archy_IsValidContinent(GetCurrentMapContinent()) then
			ArchyDigsiteListFrame_Update(self);
			if not Archy.db.char.hidden then
				ArchyDigsiteListFrame_Show(self);
			end
		else
			ArchyDigsiteListFrame_Hide(self);
		end
	
	end
end
			
function ArchyDigsiteListFrame_OnShow(self)
	DebugPrint("ArchyDigsiteListFrame_OnShow");
	Archy:UpdateDigsites();
	ArchyDigsiteListFrame_Update(self);
end
			
function ArchyDigsiteListFrame_OnHide(self)
end

function ArchyDigsiteListFrame_Show(self)
	self:Show();
end

function ArchyDigsiteListFrame_Hide(self)
	self:Hide();
end


function ArchyDigsiteListFrame_Update(self)
	if not RACES_LOADED or not Archy.hasArch then
		return;
	end
	local continentID = GetCurrentMapContinent();
	if not Archy.myDigsites[continentID] then
		return;
	end

	DebugPrint("Updating Digsite List for " .. continentID);
	for digsiteNum,digsite in ipairs(Archy.myDigsites[continentID]) do
		local name = digsite.name;
		if Archy.db.char.blacklist[name] then
			name = "|cFFFF0000" .. name;
		end
		digsiteFrame = self["digsite"..digsiteNum];
		digsiteFrame.zone:SetText(digsite.zoneName);
		digsiteFrame.digsite.name:SetText(name);
		digsiteFrame.raceID = digsite.raceID;
		digsiteFrame.raceName = digsite.raceName;
		digsiteFrame.icon:SetTexture(digsite.raceTexture);
		digsiteFrame.icon:SetSize(44,44);
		digsiteFrame:Show();
	end
end

function Archy_IsValidContinent(continentID)
	if continentID >= 1 and continentID <= 4 then
		return true
	end
	return false
end